@@ -12,13 +12,13 @@ def get_unit(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards) -> pand
         units: DataFrame generate by importing a file following schema defintion
                found in pre-alignment/workflow/schemas/units.schema.tsv
         wildcards: wildcards object with at least the following wildcard names
-               sample, type, run, and lane
+               sample, type, flowcell, and lane
     Returns:
         Series containing data of the selected row
     Raises:
         raises an exception (KeyError) if no unit can be extracted from the Dataframe
     """
-    unit = units.loc[(wildcards.sample, wildcards.type, wildcards.run, wildcards.lane)].dropna()
+    unit = units.loc[(wildcards.sample, wildcards.type, wildcards.flowcell, wildcards.lane)].dropna()
     return unit
 
 
@@ -29,7 +29,7 @@ def get_fastq_file(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards, r
         units: DataFrame generate by importing a file following schema defintion
                found in pre-alignment/workflow/schemas/units.schema.tsv
         wildcards: wildcards object with at least the following wildcard names
-               sample, type, run, and lane
+               sample, type, flowcell, and lane
         read_pair: fast1 or fastq2
     Returns:
         path for fastq file as a str
@@ -49,7 +49,7 @@ def get_fastq_adapter(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards
         units: DataFrame generate by importing a file following schema defintion
                found in pre-alignment/workflow/schemas/units.schema.tsv
         wildcards: wildcards object with at least the following wildcard names
-               sample, type, run, and lane
+               sample, type, flowcell, and lane
     Returns:
         return adapter with format seq1,seq2
     Raises:
@@ -59,6 +59,74 @@ def get_fastq_adapter(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards
     return unit["adapter"]
 
 
+def get_unit_barcode(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards) -> str:
+    """
+    function used to extract barcode for one unit(row) from units.tsv
+    Args:
+        units: DataFrame generate by importing a file following schema defintion
+               found in pre-alignment/workflow/schemas/units.schema.tsv
+        wildcards: wildcards object with at least the following wildcard names
+               sample, type, flowcell, and lane
+    Returns:
+        return barcode
+    Raises:
+        raises an exception (KeyError) if no unit can be extracted from the Dataframe
+    """
+    unit = get_unit(units, wildcards)
+    return unit["barcode"]
+
+
+def get_unit_machine(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards) -> str:
+    """
+    function used to extract machine for one unit(row) from units.tsv
+    Args:
+        units: DataFrame generate by importing a file following schema defintion
+               found in pre-alignment/workflow/schemas/units.schema.tsv
+        wildcards: wildcards object with at least the following wildcard names
+               sample, type, flowcell, and lane
+    Returns:
+        return machine id
+    Raises:
+        raises an exception (KeyError) if no unit can be extracted from the Dataframe
+    """
+    unit = get_unit(units, wildcards)
+    return unit["machine"]
+
+
+def get_unit_platform(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards) -> str:
+    """
+    function used to extract platform for one unit(row) from units.tsv
+    Args:
+        units: DataFrame generate by importing a file following schema defintion
+               found in pre-alignment/workflow/schemas/units.schema.tsv
+        wildcards: wildcards object with at least the following wildcard names
+               sample, type, flowcell, and lane
+    Returns:
+        return platform
+    Raises:
+        raises an exception (KeyError) if no unit can be extracted from the Dataframe
+    """
+    unit = get_unit(units, wildcards)
+    return unit["platform"]
+
+
+def get_unit_flowcell(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards) -> str:
+    """
+    function used to extract flowcell for one unit(row) from units.tsv
+    Args:
+        units: DataFrame generate by importing a file following schema defintion
+               found in pre-alignment/workflow/schemas/units.schema.tsv
+        wildcards: wildcards object with at least the following wildcard names
+               sample, type, flowcell, and lane
+    Returns:
+        return flowcell
+    Raises:
+        raises an exception (KeyError) if no unit can be extracted from the Dataframe
+    """
+    unit = get_unit(units, wildcards)
+    return unit["flowcell"]
+
+
 def get_units(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards, type: str = None) -> pandas.DataFrame:
     """
     function used to extract one or more units from units.tsv
@@ -78,6 +146,10 @@ def get_units(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards, type:
         files = units.loc[(wildcards.sample, wildcards.type)].dropna()
     else:
         files = units.loc[(wildcards.sample, type)].dropna()
+    if isinstance(files, pandas.Series):
+        files = pandas.DataFrame(
+            [[f[1] for f in files.iteritems()], ], columns=[f[0] for f in files.iteritems()]
+        ).set_index(units.index.names)
     return [file for file in files.itertuples()]
 
 
@@ -99,6 +171,22 @@ def get_fastq_files(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards,
     return [getattr(file, wildcards.read) for file in get_units(units, wildcards, type)]
 
 
+def get_platforms(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards) -> set:
+    """
+    function used to extract all uniq platform values for a sample and type combination found in units.tsv
+    Args:
+        units: DataFrame generate by importing a file following schema defintion
+               found in pre-alignment/workflow/schemas/units.schema.tsv
+        wildcards: wildcards object with at least the following wildcard names
+               sample, type.
+    Returns:
+        set of string
+    Raises:
+        raises an exception (KeyError) if no unit(s) can be extracted from the Dataframe
+    """
+    return set([u.platform for u in units.loc[(wildcards.sample, wildcards.type)].dropna().itertuples()])
+
+
 def get_unit_types(units: pandas.DataFrame, sample: str) -> set:
     """
     function used to extract all types of units found for a sample in units.tsv (N,T,R)
@@ -115,17 +203,17 @@ def get_unit_types(units: pandas.DataFrame, sample: str) -> set:
     return set([u.type for u in units.loc[(sample,)].dropna().itertuples()])
 
 
-def get_units_per_run(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards):
+def get_units_per_flowcell(units: pandas.DataFrame, wildcards: snakemake.io.Wildcards):
     """
-    function used to extract all sample and type combinations for one sequencing run
+    function used to extract all sample and type combinations for one sequencing flowcell
     Args:
         units: DataFrame generate by importing a file following schema defintion
                found in pre-alignment/workflow/schemas/units.schema.tsv
         wildcards: wildcards object with at least the following wildcard names
-               sample, run.
+               sample, flowcell.
     Returns:
         tuple with sample and type
     Raises:
         raises an exception (KeyError) if no unit(s) can be extracted from the Dataframe
     """
-    return set([(u.sample, u.type) for u in units[units.run == wildcards.run].itertuples()])
+    return set([(u.sample, u.type) for u in units[units.flowcell == wildcards.flowcell].itertuples()])
